
import Taro from '@tarojs/taro';
import { useState } from "./useState";
import { ResBody } from '../../types/https';
import { from } from 'rxjs';
import { finalize, pluck, tap } from 'rxjs/operators';
import { onMounted, Ref, UnwrapRef, watch } from 'vue';
import { useInjector } from './useInjector';
import { useAuthStore } from '../store/auth/useAuthStore';
import { environment } from '../../contains/environment';

interface RequestOption {
  auto?: boolean;
  method?: 'GET' | 'POST' | 'OPTIONS';
  pluck?: string[];
  successTip?: string;
  loadingTip?: string;
  type?: 'ref' | 'reactive';
  callback?: () => void;
}

export type HttpStatus = 'pending' | 'success' | 'failed' | 'error';

const defaultOption: RequestOption = {
  auto: true,
  method: 'POST',
  pluck: ['data'],
  type: 'ref',
}

// [Ref<UnwrapRef<T>>, (data?: any) => Observable<UnwrapRef<T>>, Ref<HttpStatus> ]
export function useRequest<T>(
  url: string,
  params: any = {},
  options: RequestOption = defaultOption
): [Ref<UnwrapRef<T>>, (data?: any) => void, Ref<HttpStatus> ] {

  const [ status, setStatus ]     = useState<HttpStatus>(null);
  const [ resData, setResData ]   = useState<T>(null);
  const opt                       = { ...defaultOption, ...options };

  const authInfo = useInjector(useAuthStore, 'root');

  onMounted(() => {
    if (!opt.auto) { return };
    doRequest();
  })

  watch(status, val => {
    // Taro.hideToast()
    switch (val ) {
      case 'pending':
        if (!opt.loadingTip) {return};
        Taro.showToast({
          title: opt.loadingTip,
          icon: 'loading'
        })        
      break;
      case 'success':
        if (!opt.successTip) return;        
        Taro.showToast({title: opt.successTip})
      break;
      case 'error':
      case 'failed':
        Taro.showToast({title: '请求失败', icon:'none'})
      break;
    }
  })

  const doRequest = (data: any = {}) => {
    setStatus('pending') ;    
    from(
      Taro.request({
        url: environment.baseUrl + url,
        method: opt.method,
        data: {...params, ...data, token: authInfo.token.value, uid: authInfo.user.value.id}
      })
    ).pipe(
      tap(console.log),
      tap(
        res => setStatus(res.data.success ?'success': 'failed'),
        _ => {setStatus('error'); console.log(_)}
      ),
      pluck<ResBody<T>, UnwrapRef<T>>('data', ...opt.pluck as any),
      tap(setResData),
      finalize(() => Taro.hideToast())
    ).subscribe()
  }

  return [ resData, doRequest, status ]
}
